<?php
class DWD_WC_API_Helper {
	private static $instance;
	private $user_id;
	private $api_key;
	private $http;

	private function __construct() {
		$this->user_id = get_option('dwd-wc-userid');
		$this->api_key = get_option('dwd-wc-apikey');
		$this->base = "https://api.printdeal.com";
		$this->wrapper_base = DWD_WC_WRAPPER_URL;
		$this->http = new WP_Http();

		add_action('http_api_debug', [$this, 'debug_request'], 10, 5);
	}

	public function debug_request($response = null, $type = null, $module = null, $parsed_args = null, $url = null)
	{
		if (stripos($url, "api.printdeal.com") !== false && stripos($url, DWD_WC_WRAPPER_URL) !== false) {
			return;
		}
		$http_log = [
			'time' => (new DateTime())->format(DateTime::ISO8601),
			'response' => $response,
			'type' => $type,
			'module' => $module,
			'parsed_args' => $parsed_args,
			'url' => $url
		];
		$encoded = json_encode($http_log);
		// clear out credentials
		$encoded = str_replace([json_encode(get_option('dwd-wc-apikey')), json_encode(get_option('dwd-wc-license-key'))], "### SECRET ###", $encoded);
		set_transient("dwd-wc-http-log-" . time(), $encoded, 3600);
	}

	private function request($method, $path, $body = null)
	{
		$full_url = rtrim( $this->base, '/' ) . '/' . ltrim( $path, '/' );

		$response = $this->http->request($full_url, [
			'method' => strtoupper($method),
			'headers' => [
				'Content-Type' => 'application/json',
				'User-ID' => $this->user_id,
				'API-Secret' => $this->api_key,
				'Accept' => "application/vnd.printdeal-api.v2"
			],
			'body' => $body,
			'timeout' => 30
		]);
		if ($response instanceof WP_Error) {
			throw new Exception($response->get_error_message());
		}
		if ($response['body'] === "\"Authentication failed\"") {
			throw new Exception($response['body']);
		} 
		/*if ($response['response']['code'] >= 400 && $response['response']['code'] < 600) {
			throw new Exception($response['response']['message']);
		}*/
		return json_decode($response['body'], true);
	}

	private function wrapper_request($method, $path, $body = null) {
		$full_url = rtrim( $this->wrapper_base, '/' ) . '/' . ltrim( $path, '/' );
		$response = $this->http->request($full_url, [
			'method' => strtoupper($method),
			'timeout' => 30,
	 		'headers' => apply_filters('dwd_wc_api_headers', []),
			'body' => $body
		]);
		if ($response instanceof WP_Error) {
			throw new Exception($response->get_error_message());
		}
		if ($response['response']['code'] >= 400 && $response['response']['code'] < 600) {
			throw new Exception($response['response']['message']);
		}
		return json_decode($response['body'], true);
	}

	public static function getInstance() {
		if (!self::$instance) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public function getProducts()
	{
		$products = get_transient("dwd_wc_api_products");
		if ($products) {
			return $products;
		}

		$products = $this->wrapper_request("GET", "/products");
		usort($products, function($p1, $p2) {
			return strnatcasecmp($p1['name'], $p2['name']);
		});

		set_transient("dwd_wc_api_products", $products,     3600);
		return $products;
	}

	public function getOrder($orderUuid)
	{
		return $this->request('GET', "/api/orders/{$orderUuid}");
	}

	public function createOrder($orderJson)
	{
		return $this->request('POST', "/api/orders", $orderJson);
	}

	public function getPrice($uuid, $attributes)
	{
		return $this->request('POST', "/api/products/{$uuid}", json_encode(['attributes' => $attributes]));
	}

	public function getQuantities($uuid, $attributes)
	{
		$query_params = http_build_query(array_merge(['uuid' => $uuid], $attributes));
		return $this->wrapper_request('GET', "available-quantities?$query_params");
	}

	public function getAvailableOptions($uuid, $attributes)
	{
		$query_params = http_build_query(array_merge(['uuid' => $uuid], $attributes));
		return $this->wrapper_request('GET', "available-options?$query_params");
	}

	public function getAvailableRanges($uuid, $attributes)
	{
		$query_params = http_build_query(array_merge(['uuid' => $uuid], $attributes));
		return $this->wrapper_request('GET', "available-ranges?$query_params");
	}

	public function validateOrder($order_lines) {
		try {
			$this->wrapper_request("POST", "/order/validate", json_encode($order_lines));
			return true;
		} catch (Exception $e) {
			return false;
		}
	}

	public function validateLicense($license, $url) {
		try {
			$this->wrapper_request("POST", "/license/validate", json_encode(['license' => $license, 'website' => $url]));
			return true;
		} catch (Exception $e) {
			return false;
		}
	}
	/**
	 * This function will validate the credentials and check whether these can be used for ordering
	 * return values can be 'valid', 'invalid' or 'test'
	 */
	public function validateCredentials($userId, $apiKey) {
		$response = null;
		try {
			// take the userid/apikey from the credentials
			$this->user_id = $userId;
			$this->api_key = $apiKey;
			$api_response = $this->request('GET', '/api/orders');
			if ($api_response === null) {
				$response = "test";
			} else {
				$response = "valid";
			}
		} catch (Exception $error) {
			$response = "invalid";
		}
		// restore the userid/apikey
		$this->user_id = get_option('dwd-wc-userid');
		$this->api_key = get_option('dwd-wc-apikey');
		return $response;
	}
}